import { DataStreamTransportManagement } from "../gen/HomeKit-DataStream";
import { DataStreamServerEventMap, GlobalEventHandler, GlobalRequestHandler } from "./DataStreamServer";
import { Event } from "../EventEmitter";
export declare enum TransferTransportConfigurationTypes {
    TRANSFER_TRANSPORT_CONFIGURATION = 1
}
export declare enum TransportTypeTypes {
    TRANSPORT_TYPE = 1
}
export declare enum SetupDataStreamSessionTypes {
    SESSION_COMMAND_TYPE = 1,
    TRANSPORT_TYPE = 2,
    CONTROLLER_KEY_SALT = 3
}
export declare enum SetupDataStreamWriteResponseTypes {
    STATUS = 1,
    TRANSPORT_TYPE_SESSION_PARAMETERS = 2,
    ACCESSORY_KEY_SALT = 3
}
export declare enum TransportSessionConfiguration {
    TCP_LISTENING_PORT = 1
}
export declare enum TransportType {
    HOMEKIT_DATA_STREAM = 0
}
export declare enum SessionCommandType {
    START_SESSION = 0
}
export declare enum DataStreamStatus {
    SUCCESS = 0,
    GENERIC_ERROR = 1,
    BUSY = 2
}
export declare class DataStreamManagement {
    private readonly dataStreamServer;
    dataStreamTransportManagementService: DataStreamTransportManagement;
    supportedDataStreamTransportConfiguration: string;
    lastSetupDataStreamTransportResponse: string;
    constructor();
    /**
     * @returns the DataStreamTransportManagement service
     */
    getService(): DataStreamTransportManagement;
    /**
     * Registers a new event handler to handle incoming event messages.
     * The handler is only called for a connection if for the give protocol no ProtocolHandler
     * was registered on the connection level.
     *
     * @param protocol {string | Protocols} - name of the protocol to register the handler for
     * @param event {string | Topics} - name of the event (also referred to as topic. See {Topics} for some known ones)
     * @param handler {GlobalEventHandler} - function to be called for every occurring event
     */
    onEventMessage(protocol: string, event: string, handler: GlobalEventHandler): this;
    /**
     * Removes an registered event handler.
     *
     * @param protocol {string | Protocols} - name of the protocol to unregister the handler for
     * @param event {string | Topics} - name of the event (also referred to as topic. See {Topics} for some known ones)
     * @param handler {GlobalEventHandler} - registered event handler
     */
    removeEventHandler(protocol: string, event: string, handler: GlobalEventHandler): this;
    /**
     * Registers a new request handler to handle incoming request messages.
     * The handler is only called for a connection if for the give protocol no ProtocolHandler
     * was registered on the connection level.
     *
     * @param protocol {string | Protocols} - name of the protocol to register the handler for
     * @param request {string | Topics} - name of the request (also referred to as topic. See {Topics} for some known ones)
     * @param handler {GlobalRequestHandler} - function to be called for every occurring request
     */
    onRequestMessage(protocol: string, request: string, handler: GlobalRequestHandler): this;
    /**
     * Removes an registered request handler.
     *
     * @param protocol {string | Protocols} - name of the protocol to unregister the handler for
     * @param request {string | Topics} - name of the request (also referred to as topic. See {Topics} for some known ones)
     * @param handler {GlobalRequestHandler} - registered request handler
     */
    removeRequestHandler(protocol: string, request: string, handler: GlobalRequestHandler): this;
    /**
     * Forwards any event listener for an DataStreamServer event to the DataStreamServer instance
     *
     * @param event - the event to register for
     * @param listener - the event handler
     */
    onServerEvent(event: Event<keyof DataStreamServerEventMap>, listener: DataStreamServerEventMap[Event<keyof DataStreamServerEventMap>]): this;
    private handleSetupDataStreamTransportWrite;
    private static buildSetupStatusResponse;
    private buildSupportedDataStreamTransportConfigurationTLV;
    private setupService;
}
//# sourceMappingURL=DataStreamManagement.d.ts.map