/// <reference types="node" />
import { Status } from './lib/HAPServer';
import { Characteristic, CharacteristicProps } from './lib/Characteristic';
export declare type Nullable<T> = T | null;
export declare type WithUUID<T> = T & {
    UUID: string;
};
export interface ToHAPOptions {
    omitValues: boolean;
}
export declare type Callback = (...args: any[]) => void;
export declare type NodeCallback<T> = (err: Nullable<Error> | undefined, data?: T) => void;
export declare type VoidCallback = (err?: Nullable<Error>) => void;
export declare type PairingsCallback<T> = (err: number, data?: T) => void;
export declare type PrimitiveTypes = string | number | boolean;
declare type HAPProps = Pick<CharacteristicProps, 'perms' | 'format' | 'description' | 'unit' | 'maxValue' | 'minValue' | 'minStep' | 'maxLen'> & Pick<Characteristic, 'valid-values' | 'valid-values-range'>;
export declare type HapCharacteristic = HAPProps & {
    iid: number;
    type: string;
    value: string | number | {} | null;
};
export declare type CharacteristicValue = PrimitiveTypes | PrimitiveTypes[] | {
    [key: string]: PrimitiveTypes;
};
export declare type CharacteristicChange = {
    newValue: CharacteristicValue;
    oldValue: CharacteristicValue;
    context?: any;
    characteristic: Characteristic;
};
export declare type HapService = {
    iid: number;
    type: string;
    characteristics: HapCharacteristic[];
    primary: boolean;
    hidden: boolean;
    linked: number[];
};
export declare type CharacteristicData = {
    aid: number;
    iid: number;
    v?: string;
    value?: string;
    s?: Status;
    status?: Status;
    e?: string;
    ev?: boolean;
    r?: boolean;
};
export declare type AudioCodec = {
    samplerate: number;
    type: string;
};
export declare type VideoCodec = {
    levels: number[];
    profiles: number[];
};
export declare type Source = {
    port: number;
    srtp_key: Buffer;
    srtp_salt: Buffer;
    targetAddress?: string;
    proxy_rtp?: number;
    proxy_rtcp?: number;
};
export declare type Address = {
    address: string;
    type: 'v4' | 'v6';
};
export declare type AudioInfo = {
    codec: string | number;
    channel: number;
    bit_rate: number;
    sample_rate: number;
    packet_time: number;
    pt: number;
    ssrc: number;
    max_bit_rate: number;
    rtcp_interval: number;
    comfort_pt: number;
};
export declare type VideoInfo = {
    profile: number;
    level: number;
    width: number;
    height: number;
    fps: number;
    ssrc: number;
    pt: number;
    max_bit_rate: number;
    rtcp_interval: number;
    mtu: number;
};
export declare type SessionIdentifier = Buffer | string;
export declare type StreamAudioParams = {
    comfort_noise: boolean;
    codecs: AudioCodec[];
};
export declare type Resolution = [number, number, number];
export declare type StreamVideoParams = {
    codec?: VideoCodec;
    resolutions: Resolution[];
};
export {};
//# sourceMappingURL=types.d.ts.map